#include <stdio.h>

#include "list-6c.c"

typedef struct __hash hash_t;
typedef list_iterator_t hash_iterator_t;

struct __hash {
    int num_buckets;
    list_t* buckets;
};

void hash_init(hash_t* h, int num_buckets) {
    h->num_buckets = num_buckets;
    h->buckets = malloc(sizeof(list_t) * num_buckets);
    assert(h->buckets != NULL);
    int i;
    for (i = 0; i < num_buckets; i++) 
	list_init(&(h->buckets[i]));
}

void hash_fini(hash_t* h) {
    int i;
    for (i = 0; i < h->num_buckets; i++)
        list_fini(&(h->buckets[i]));
    free(h->buckets);
}

void hash_insert(hash_t* h, int key, void* value) {
    list_insert_front(&(h->buckets[key % h->num_buckets]), key, value);
}

int hash_delete(hash_t* h, int key, void** value) {
    return list_delete(&(h->buckets[key % h->num_buckets]), key, value);
}

int hash_lookup(hash_t* h, int key, void** value) {
    return list_lookup(&(h->buckets[key % h->num_buckets]), key, value);
}

void hash_apply(hash_t* h, hash_iterator_t func) {
    int i;
    for (i = 0; i < h->num_buckets; i++)
	list_apply(&(h->buckets[i]), func);
}


//
// main code
//
void my_printer(node_t* n) {
    printf("contents %d %lld\n", n->key, (long long)n->value);
}


int main(int argc, char *argv[]) {
    hash_t H;
    hash_init(&H, 17471);

    long long int i;
    for (i = 0; i < 1000; i++) 
	hash_insert(&H, i, (void*)(i+1));

    hash_apply(&H, my_printer);

    for (i = 0; i < 1000; i++) {
	long long v;
	int rc = hash_lookup(&H, i, (void*) &v);
	printf("lookup %d %lld\n", rc, v);
    }

    for (i = 0; i < 1000; i++) {
	long long v;
	int rc = hash_delete(&H, i, (void*) &v);
	printf("delete %d %lld\n", rc, v);
    }

    for (i = 0; i < 1000; i++) {
	long long v = -1;
	int rc = hash_delete(&H, i, (void*) &v);
	printf("delete %d %lld\n", rc, v);
    }

    printf("empty?\n");
    hash_apply(&H, my_printer);

    return 0;
}

